/**
 * @author Shun Moriya <shun126@users.sourceforge.jp> 
 */

import java.applet.Applet;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Event;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.MediaTracker;
import java.io.*;
import java.net.URL;

import poly.Material;
import poly.plugins.mqo.*;

public class Main extends Applet implements Runnable
{
	Thread threadThis = null;			// Thread for refreshing image
	Image offscreenImage;				// offscreen Image
	Graphics offscreen;					// offscreen Graphics
	Dimension screenSize;				// Size of screen
	poly.Scene scene = null;			// Render scene

	final float vUp[] = new float[4];
	final float vTarget[] = new float[4];
	float cameraMatrix[] = new float[4*4];
	float cameraTranslate[] = new float[4];
	float cameraRotate[] = new float[4];

	final int cameraRotateWait = 30 * 3;
	int cameraRotateWaitCounter;
	
	int pre_x = 160;
	int pre_y = 120;
	float phi   = 0.0f;
	float theta = 0.0f;
	float omega = 0.0f;
	float sense = 0.01f;
	float fps = 0;
	int count = 59;
	long time = System.currentTimeMillis();
	
	public void init()
	{
		setLayout(null);
		setSize(300, 300);
		screenSize = getSize();
		cameraRotateWaitCounter = 0;
		try{
			screenSize.width = Integer.parseInt(getParameter("width"));
			screenSize.height = Integer.parseInt(getParameter("height"));
		}catch (Exception e){
			screenSize.width = 640;
			screenSize.height = 480;
		}
		float length;
		try{
			length = Float.valueOf(getParameter("length")).floatValue();
		}catch(Exception e){
			length = 500;
		}
		poly.Graphic.setVector(cameraTranslate, 0, 100, length);
		resize(screenSize.width, screenSize.height);

		offscreenImage = createImage(screenSize.width, screenSize.height);
		offscreen = offscreenImage.getGraphics();

		poly.Graphic.init(screenSize.width, screenSize.height, true, true, true);

		try{
			scene = new poly.Scene();

			String filename = getParameter("model");
			System.out.println("loading model file ... " + filename);
			InputStream is = new URL(getCodeBase(), filename ).openStream();
			InputStreamReader myin = new InputStreamReader( is );
			
			Mqo mqoPlugin = new Mqo();
			mqoPlugin.importFromFile(scene, new BufferedReader(myin));

			// loading texture files			
			for(int i = 0; i < scene.materials.size(); i++)
			{
				Material material = (Material)scene.materials.elementAt(i);
				filename = material.texture;
				if(filename != null)
				{
					Image textureImage;
					URL url_webcam;
					MediaTracker mediaTracker_track;
						
					System.out.println("loading texture file ... " + filename);
					
					url_webcam = Main.class.getResource(filename);
					textureImage = getImage(url_webcam);
					mediaTracker_track = new MediaTracker(this);
					mediaTracker_track.addImage(textureImage, 1);
					mediaTracker_track.waitForID(1);
					
					if(textureImage.getWidth(null) <= 0 || textureImage.getHeight(null) <= 0)
						throw new Exception("File not found: " + filename);
					
					poly.Graphic.setTexture(i, textureImage);
				}
			}

		}catch(IOException e){
			String errorMessage = "Can't initialize scene data :" + e.toString();
			System.out.println(errorMessage);
			showStatus(errorMessage);
			return;
		}catch(InterruptedException e){
			String errorMessage = "Problem getting initial texture: "+e.toString();
			System.out.println(errorMessage);
			showStatus(errorMessage);
			return;
		}catch(Exception e){
			String errorMessage = "Problem getting initial image: "+e.toString();
			System.out.println(errorMessage);
			showStatus(errorMessage);
			return;
		}

		// set light position
		poly.Graphic.setLightPosition(30, -10, 0);
		poly.Graphic.setLightColor(0.9f, 0.9f, 0.9f);
		poly.Graphic.setAmbientColor(0.5f, 0.5f, 0.5f);
		poly.Graphic.setSpeculerColor(1.0f, 1.0f, 1.0f);
	}

	public void destroy()
	{
		threadThis = null;
	}

	public void start()
	{
		// Create and start the new thread initially suspended
		if(threadThis == null)
		{
			threadThis = new Thread(this);
			threadThis.start();
		}
	}

	public void stop()
	{
		destroy();
	}

	public void run()
	{
		Thread thread_Current = Thread.currentThread();
		while(threadThis == thread_Current)
		{
			try{
				if(cameraRotateWaitCounter > 0)
				{
					cameraRotateWaitCounter--;
				}else{
					cameraRotate[1] += 0.02;
				}
				
				// Repaint the image
				repaint();

				// Put the refresh thread to sleep for the supplied interval
				Thread.sleep(1000 / 30);
				
			}catch (Exception e){
				showStatus("Problem getting run: "+e.toString());
				stop();
			}
		}
		destroy();
	}

	public void update(Graphics g)
	{
		paint(g);
	}

	public void paint(Graphics g)
	{
		poly.Graphic.beginScene();

		// Js̏Ɛݒ
//		poly.Graphic.setVector(vUp, 0, 1, 0);
//		poly.Graphic.setVector(vTarget, 0, 0, 0);
		poly.Graphic.setViewPosition(cameraTranslate[0], cameraTranslate[1], cameraTranslate[2]);

		poly.Graphic.unitMatrix(cameraMatrix);
		poly.Graphic.setTransMatrix(cameraMatrix, cameraTranslate);
		poly.Graphic.rotMatrixYXZ(cameraMatrix, cameraMatrix, cameraRotate);
		poly.Graphic.setTransform(poly.Graphic.TRANSFORM_VIEW, cameraMatrix);
//		poly.Graphic.setTransform(poly.Graphic.TRANSFORM_VIEW, poly.Graphic.lookat(cameraTranslate, vTarget, vUp));

		poly.Graphic.clear();
		scene.translate();
		poly.Graphic.endScene(this, offscreen);

		// print debug infomations
		printFrameRate(offscreen, 10, 16 + 0);
/*
		offscreen.setColor(Color.black);
		offscreen.drawString("phi:" + phi + " / theta:" + theta, 10, 16 + 15);
		offscreen.drawString("camera("+cameraTranslate[0]+","+cameraTranslate[1]+","+cameraTranslate[2]+")", 10, 16 + 30);
*/

		// draw screen from offscreen(back buffer)
		g.drawImage(offscreenImage, 0, 0, this);
	}

	public boolean mouseDown(Event evt, int x, int y)
	{
		pre_x = x;
		pre_y = y;
		
		cameraRotateWaitCounter = cameraRotateWait;
		return(true);
	}
	public boolean mouseUp(Event evt, int x, int y)
	{
		cameraRotateWaitCounter = cameraRotateWait;
		return(true);
	}
	public boolean mouseDrag(Event evt, int x, int y)
	{
		int dx = x - pre_x;
		int dy = y - pre_y;

		if(evt.controlDown())
		{
			cameraTranslate[2] -= dy;
		}else if(evt.shiftDown()){
			cameraTranslate[0] += dx;
			cameraTranslate[1] += dy;
		}else{
			phi    = dx * sense;
			theta  = dy * sense;
			cameraRotate[0] += theta;
			cameraRotate[1] -= phi;
			cameraRotate[3] = 1.0f;
		}
		pre_x  = x;
		pre_y  = y;
		
		cameraRotateWaitCounter = cameraRotateWait;
		return(true);
	}
	public void printFrameRate(Graphics g, int x, int y)
	{
		if(++ count == 60)
		{
			fps   = 1000 / (float)((System.currentTimeMillis() - time) / 60);
			time  = System.currentTimeMillis();
			count = 0;
		}
		g.setColor(Color.black);
		g.drawString("FPS " + fps, x, y);
	}
}
